﻿using UnityEngine;
using UnityEngine.SceneManagement;
using System.Collections.Generic;
using Sys = System;

public enum Act
{
    Stand = 0,
    Sing = 1,
    Slogan = 2,
    Clap = 3,
}

public class ActCmd
{
    public Act act;
    public int round;
    public float spf;
    
    public ActCmd(Act act, int round, float spf)
    {
        this.act = act;
        this.round = round;
        this.spf = spf;
    }
}

public class Game : Singleton<Game> {

    enum State
    {
        None,
        SpeechTurn,
        OfficialTurn,
        CrowdTurn,
        EndTurn,
    }
    
    State state = State.None;

    List<PersonController> peopleList = new List<PersonController>();
    List<PersonController> officialList = new List<PersonController>();
    Player player;
    KimController kim;
    
    float speechDuration = 3.0f;
    
    float speechTimer = 0.0f;
    
    AudioSource crowdAudioSrc;
    AudioSource officialsAudioSrc;
    AudioSource playerAudioSrc; 
    
    int chance = 3;
    
    List<ActCmd> actCmdList = new List<ActCmd>();
    
    ActCmd standCmd = new ActCmd(Act.Stand, 1, 1.0f);
    
    int turnCount = 1;
    
	void Start () {
	
        Transform crowd = GameObject.Find("Crowd").transform;
        for(int i=0; i<crowd.childCount; i++) {
            Transform child = crowd.GetChild(i);
            peopleList.Add(child.GetComponent<PersonController>());
        }
        
        crowdAudioSrc = crowd.GetComponent<AudioSource>();
        
        Transform officials = GameObject.Find("Officials").transform;
        for(int i=0; i<officials.childCount; i++) {
            Transform child = officials.GetChild(i);
            officialList.Add(child.GetComponent<PersonController>());
        }
        
        officialsAudioSrc = crowd.GetComponent<AudioSource>();
        
        kim = GameObject.Find("Kim").GetComponent<KimController>();
        player = GameObject.Find("Player").GetComponent<Player>();
        
        playerAudioSrc = player.GetComponent<AudioSource>();
        
        actCmdList = GenerateRandomActCmdList();
        
        FuncRunner.Instance.Run(1.0f, () => {
            EnterSpeech();
        });
	}
	
    bool officialCmdStarted = false;
    bool crowdCmdStarted = false;
    
	// Update is called once per frame
	void Update () {
	
        if(state == State.SpeechTurn) {
            // is speed done?
            speechTimer += Time.deltaTime;
            if(speechTimer + 0.3f > speechDuration) {
                
                SoundManager.Instance.StopSpeech();

                if(speechTimer > speechDuration) {
                    state = State.OfficialTurn;
                    speechTimer = 0.0f;
                    kim.StopTalking();
                }
            }
        }
        else if(state == State.OfficialTurn) {
            
            HandlePlayerInput();
            
            if(actCmdList.Count > 0 && !officialCmdStarted ) {
                Debug.Log("cmd " + actCmdList[0].act);
                officialCmdStarted = true;
                
                ActCmd refCmd = actCmdList[0]; 
                ActCmd cmd = new ActCmd(refCmd.act, refCmd.round+2, refCmd.spf);
                
                Act actType = cmd.act;
                
                IssueActCmd(officialList, cmd, () => {
                    if(actType == Act.Clap)
                        PlayCrowdClapping(1.0f);
                    else if(actType == Act.Slogan) 
                        PlayCrowdChanting(1.0f);
                    else if(actType == Act.Sing)
                        PlayCrowdSing(1.0f);
                }, 
                null);
                FuncRunner.Instance.Run(refCmd.spf*2*2, () => {
                    
                    Debug.Log("to crowd turn");
                    state = State.CrowdTurn;
                    officialCmdStarted = false;
                });
            }
        }
        else if(state == State.CrowdTurn) {
            
            HandlePlayerInput();
            
            if(actCmdList.Count > 0 && !crowdCmdStarted) {
                    
                ActCmd cmd = actCmdList[0];
                crowdCmdStarted = true;
                
                turnCmdTimestamp = Time.time;
                turnCmd = cmd;
                
                Act actType = cmd.act;
                
                IssueActCmd(peopleList, cmd, 
                null,
                () => {           
                    actCmdList.RemoveAt(0);
                    state = State.EndTurn;
                    Debug.Log("setting end state");
                    player.ResetPose();
                    
                    IssueActCmd(officialList, standCmd, null, null);
                    IssueActCmd(peopleList, standCmd, () => {
                        PlayCrowdStand(1.0f);
                    }, null);
                    crowdCmdStarted = false;
                });
            }
        }
        else if(state == State.EndTurn) {
            
            bool success = true;
            
            Debug.Log("in end turn state");
            
            if(!IsInSync(turnCmd, turnCmdTimestamp, inputList)) {
                chance--;
                success = false;
            }
            
            state = State.None;

            if(chance == 0) {
                kim.ShowFace(KimController.Face.Rage);    

                FuncRunner.Instance.Run(1.0f, () => {
                    // load death scene();
                    
                    PlayPlayerAlarm(1.0f);
                    LightingManager.Instance.TurnOffMainLight();
                    LightingManager.Instance.ShowDeathLight();
                    FuncRunner.Instance.Run(3.0f, () => {
                        Debug.Log("GAME END");
                        SceneManager.LoadScene("GameOver");
                    });
                });
            }
            else {
                
                if(success){
                    kim.ShowFace(KimController.Face.Happy);
                }
                else {
                    kim.ShowFace(KimController.Face.Angry);
                }

                turnCmd = null;
                turnCmdTimestamp = 0.0f;
                inputList.Clear();

                FuncRunner.Instance.Run(1.5f, () => {
                    
                    turnCount++;
                    EnterSpeech();
                });
            }
        }
        else if(state == State.None)
        {
            // no nothing
        }
	}
    
    static bool IsInSync(ActCmd cmd, float baseRefTime, List<InputRecord> inputs)
    {
        int totalScore = cmd.round*2;
        int score = 0;
        
        for(int i=0; i<Mathf.Min(inputs.Count, cmd.round*2); i++)
        {
            float refTime = baseRefTime + i*cmd.spf;
            
            InputRecord r = inputs[i];
            if(r.act == cmd.act) {
            
                float offTime = Mathf.Abs(refTime - r.timestamp);
                if(offTime < cmd.spf*1.5f) {
                    score++;
                }    
            }
        }
        
        Debug.Log("score " + score + ", total score " + totalScore);
        
        if(score/(float)totalScore >= 0.5 || 
            score >= totalScore-1) {
            return true;
        }
        
        return false;
    }
    
    void EnterSpeech()
    {
        kim.StartTalking();
        state = State.SpeechTurn;
    }
    
    void IssueActCmd(List<PersonController> list, ActCmd cmd, Sys.Action sfx, Sys.Action onComplete)
    {    
        for(int i=0; i<list.Count; i++) {
            PersonController o = list[i];
            if(i == 0)
                o.Do(cmd.act, cmd.round, sfx, onComplete);
            else 
                o.Do(cmd.act, cmd.round, null, null);
        }
    }
    
    ActCmd GenerateRandomActCmd()
    {
        int randRound = Random.Range(3, 6);
        
        float minSpf = 1.0f / 4 / (1.05f*Mathf.Min(20, turnCount));
        float maxSpf = 1.0f / 4;
        float spf = Random.Range(minSpf, maxSpf);
        ActCmd cmd = new ActCmd((Act)Random.Range(1,4), randRound, spf);

        return cmd;
    }
    
    List<ActCmd> GenerateRandomActCmdList()
    {
        List<ActCmd> list = new List<ActCmd>();
        int count = 30;
        while(count > 0) {
            list.Add( GenerateRandomActCmd() );
            count--;
        }
        
        return list;
    }
    
    class InputRecord
    {
        public Act act;
        public float timestamp;
        public bool isDown;
        
        public InputRecord(Act act, float timestamp, bool isDown){
            this.act = act;
            this.timestamp = timestamp;
            this.isDown = isDown;
        }
    };
    
    float turnCmdTimestamp = 0.0f;
    ActCmd turnCmd = null;
    List<InputRecord> inputList = new List<InputRecord>();
    
    public void HandlePlayerInput()
    {
        if(Input.GetKeyDown(KeyCode.A)) {
           player.PressPose(Act.Stand);
        }
        else if(Input.GetKeyUp(KeyCode.A)) {
        }
        
        else if(Input.GetKeyDown(KeyCode.S)) {
            player.PressPose(Act.Clap);
            PlayPlayerClap(1.0f);
            inputList.Add(new InputRecord(Act.Clap, Time.time, true));
        }
        else if(Input.GetKeyUp(KeyCode.S)) {
            player.ReleasePose(Act.Clap);
            inputList.Add(new InputRecord(Act.Clap, Time.time, false));
        }
        
        else if(Input.GetKeyDown(KeyCode.D)) {
            player.PressPose(Act.Slogan);
            PlayPlayerChant(1.0f);
            inputList.Add(new InputRecord(Act.Slogan, Time.time, true));
        }
        else if(Input.GetKeyUp(KeyCode.D)) {
            player.ReleasePose(Act.Slogan);
            inputList.Add(new InputRecord(Act.Slogan, Time.time, false));
        }
        
        else if(Input.GetKeyDown(KeyCode.F)) {
            player.PressPose(Act.Sing);
            inputList.Add(new InputRecord(Act.Sing, Time.time, true));
        }
        else if(Input.GetKeyUp(KeyCode.F)) {
            player.ReleasePose(Act.Sing);
            inputList.Add(new InputRecord(Act.Slogan, Time.time, false));
        }
        
        if(Input.GetKeyDown(KeyCode.Space)) {
            LightingManager.Instance.TurnOffMainLight();
            LightingManager.Instance.ShowDeathLight();
        }
    }
    
    
    public void PlayCrowdChanting(float speedScale)
    {
        crowdAudioSrc.clip = SoundManager.Instance.crowdChant;
        crowdAudioSrc.Play();
    }
    
    public void PlayCrowdClapping(float speedScale)
    {
        crowdAudioSrc.clip = SoundManager.Instance.crowdClap;
        crowdAudioSrc.Play();
    }
    
    public void PlayCrowdSing(float speedScale)
    {
        crowdAudioSrc.clip = SoundManager.Instance.crowdHey;
        crowdAudioSrc.Play();
    }
    
    public void PlayCrowdStand(float speedScale)
    {
        crowdAudioSrc.clip = SoundManager.Instance.stand;
        crowdAudioSrc.Play();
    }
    
    public void PlayPlayerClap(float speedScale)
    {
        playerAudioSrc.clip = SoundManager.Instance.playerClap;
        playerAudioSrc.Play();
    }
    
    public void PlayPlayerChant(float speedScale)
    {
        playerAudioSrc.clip = SoundManager.Instance.playerChant;
        playerAudioSrc.Play();
    }
    
    public void PlayPlayerHey(float speedScale)
    {
        playerAudioSrc.clip = SoundManager.Instance.playerHey;
        playerAudioSrc.Play();
    }
    
    public void PlayPlayerAlarm(float speedScale)
    {
        playerAudioSrc.clip = SoundManager.Instance.alarm;
        playerAudioSrc.Play();
    }
}
