﻿using UnityEngine;
using System.Collections;
using DG.Tweening;

public class KimController : MonoBehaviour {

    public Mesh NormalFaceMesh;
    public Mesh HappyFaceMesh;
    public Mesh VeryHappyFaceMesh;
    public Mesh AngryFaceMesh;
    public Mesh RageFaceMesh;
    
    public Mesh[] TalkFaceMesh = new Mesh[2];

    public enum Face
    {
        Normal,
        Happy,
        VeryHappy,
        Angry,
        Rage,
        Talk,
    }

    Transform view;
    MeshFilter meshRef;
    
    Face faceState = Face.Normal;
    float faceTimer = 0.0f;
    
    float talkFaceSpf = 1.0f/8.0f;
    bool talkFaceSwitch = false;

	// Use this for initialization
	void Start () {
	   view = transform.Find("kim_face/default");
       meshRef = view.GetComponentInChildren<MeshFilter>();
	}
	
	// Update is called once per frame
	void Update () {

        //HanldeTestInput();

        if(faceState == Face.Talk) {
            faceTimer += Time.deltaTime;
            
            if(faceTimer >= talkFaceSpf) {
                faceTimer -= talkFaceSpf;
                if(talkFaceSwitch) {
                    meshRef.mesh = TalkFaceMesh[1];
                    talkFaceSwitch = false;
                }
                else {
                    meshRef.mesh = TalkFaceMesh[0];
                    talkFaceSwitch = true;
                }
            }
        }
	}
    
    public void ShowFace(Face face)
    {   
        faceState = face;
        faceTimer = 0.0f;
        
        switch(face) {
            case Face.Normal : {
                meshRef.mesh = NormalFaceMesh;
                break;
            }
            case Face.Angry : {
                meshRef.mesh = AngryFaceMesh;
                break;
            }
            case Face.Happy : {
                meshRef.mesh = HappyFaceMesh;
                break;
            }
            case Face.Rage : {
                meshRef.mesh = RageFaceMesh;
                break;
            }
            case Face.VeryHappy : {
                meshRef.mesh = VeryHappyFaceMesh;
                break;
            }
            case Face.Talk : {
                meshRef.mesh = TalkFaceMesh[0];
                break;
            }
        }
    }
    
    Tween shakeTween = null;
    
    public void StartTalking()
    {
        SoundManager.Instance.StartSpeech();
        ShowFace(Face.Talk);
        
        float oy = view.transform.position.y;
        shakeTween = DOTween.Sequence().Make(
            view.DOMoveY(oy+0.2f, 0.5f),
            view.DOMoveY(oy, 0.5f)
        ).SetLoops(-1);
    }
    
    public void StopTalking()
    {
        SoundManager.Instance.StopSpeech();
        ShowFace(Face.Normal);
        if(shakeTween != null) {
            shakeTween.Kill();
            shakeTween = null;
        }
    }
    
    void HanldeTestInput()
    {
        if(Input.GetKeyDown(KeyCode.Alpha1) ){
            ShowFace(Face.Normal);
        }
        else if(Input.GetKeyDown(KeyCode.Alpha2)) {
            ShowFace(Face.Happy);
        }
        else if(Input.GetKeyDown(KeyCode.Alpha3)) {
            ShowFace(Face.VeryHappy);
        }
        else if(Input.GetKeyDown(KeyCode.Alpha4)) {
            ShowFace(Face.Angry);
        }
        else if(Input.GetKeyDown(KeyCode.Alpha5)) {
            ShowFace(Face.Rage);
        }
        else if(Input.GetKeyDown(KeyCode.Alpha6)) {
            ShowFace(Face.Talk);
        }
    }
}
