﻿using UnityEngine;
using System.Collections;
using Sys = System;

public class PersonController : MonoBehaviour {
    
    public Mesh SitMesh;
    public Mesh StandMesh;
    public Mesh[] SingMesh = new Mesh[2];
    public Mesh[] SloganMesh = new Mesh[2];
    public Mesh[] ClapMesh = new Mesh[2];

    Transform view;
    MeshFilter meshRef;
    
    int frame = 0;
    
    Act actState;
    float actTimer = 0.0f;
    
    float singSpf = 1.0f/4;
    float sloganSpf = 1.0f/4;
    float clapSpf = 1.0f/4;

    int times = 0;
    int maxTimes = 1;

	// Use this for initialization
	void Start () {
	   view = transform.Find("default");
       meshRef = view.GetComponent<MeshFilter>();
	}
	
    public void SetPose(Act act, int frame)
    {
        switch(act) {
            case Act.Clap: {
                meshRef.mesh = ClapMesh[frame]; 
                break;
            }
            case Act.Stand: {
                meshRef.mesh = StandMesh;
                break;
            }
            case Act.Slogan: {
                meshRef.mesh = SloganMesh[frame];
                break;
            }
            case Act.Sing: {
                meshRef.mesh = SingMesh[frame];
                break;
            }
        }
    }
    
    Coroutine runningCoro = null;
    
    public void Do(Act act, int times, Sys.Action sfx, Sys.Action onComplete)
    {
        if(runningCoro != null) {
            StopCoroutine(runningCoro);
        }
        
        runningCoro = StartCoroutine(PlayPose(act, times, 1.0f / 4.0f, sfx, onComplete));
    }

    IEnumerator PlayPose(Act act, int times, float spf, Sys.Action sfx, Sys.Action onComplete = null)
    {
        while(times > 0) {
            
            if(sfx != null) {
                sfx();
            }
            
            SetPose(act, 1);
            yield return new WaitForSeconds(spf);
            SetPose(act, 0);
            yield return new WaitForSeconds(spf);
            times--;
        }
        
        if(onComplete != null)
            onComplete();
    }
    
    public void PressPose(Act act)
    {
        SetPose(act, 1);
    }
    
    public void ReleasePose(Act act)
    {
        SetPose(act, 0);
    }
}
